@@ -213,7 +213,7 @@ class MfgBase(object):
         if n == 0:
             return (0, 1)
         else:
-            return (1 - 1 / (n+1), 1 / (n+1))
+            return (1 - 1 / (n + 1), 1 / (n + 1))
 
     def solve(self, maxit=10, fp=False, pn=default_fp_pn):
         # fp: bool, use fictious play or not
@@ -221,7 +221,7 @@ class MfgBase(object):
         self.diff_M_Mbar = []
         for i in range(0, maxit):
             # solve FP with Q
-            Q = (self.QL, self.QR)
+            Q = [self.QL, self.QR]
             self.solve_FP(Q)
             # update M_bar if use fictious play
             if fp:
@@ -245,7 +245,7 @@ class MfgBase(object):
         return
 
     def default_spi1_pn(n):
-        return (n / (n + 2), 1 / (n + 2))
+        return ((n + 1) / (n + 2), 1 / (n + 2))
 
     def solve_with_spi1(self, maxit=10, pn=default_spi1_pn):
         self.diff_M = []
@@ -254,12 +254,12 @@ class MfgBase(object):
         QR_bar = np.copy(self.QR)
         for i in range(0, maxit):
             # solve FP with Q_bar
-            Q_bar = (QL_bar, QR_bar)
+            Q_bar = [QL_bar, QR_bar]
             self.solve_FP(Q_bar)
             # update U with M and Q_bar
             self.solve_HJB(self.M, Q_bar)
             # update Q with U and M
-            self.update_Q(self.U, self.M_bar)
+            self.update_Q(self.U, self.M)
             # update Q_bar
             p1, p2 = pn(i)
             QL_bar = p1 * QL_bar + p2 * self.QL
@@ -276,7 +276,7 @@ class MfgBase(object):
         if n == 0:
             return (0, 1)
         else:
-            return (1 - 1 / (n+1), 1 / (n+1))
+            return (1 - 1 / (n + 1), 1 / (n + 1))
 
     def solve_with_spi2(self, maxit=10, pn=default_spi2_pn):
         self.diff_M = []
@@ -396,7 +396,7 @@ class MfgBase(object):
             ql = np.maximum((self.DL @ u) / self.F1_func(m), 0.0)
             qr = np.minimum((self.DR @ u) / self.F1_func(m), 0.0)
             q2 = ql**2 + qr**2
-            cost += np.sum(self.dt * self.dx * m * 0.5 * self.F1_func(m) * q2)
+            cost += np.sum(self.dt * self.dx * m * (0.5 * self.F1_func(m) * q2 + self.F2_func(m)))
         cost += np.sum(self.dx * self.UT * self.M[-1, :])
         return cost
 
@@ -625,7 +625,7 @@ class MfgBase2D(object):
         if n == 0:
             return (0, 1)
         else:
-            return (1 - 1 / (n+1), 1 / (n+1))
+            return (1 - 1 / (n + 1), 1 / (n + 1))
 
     def solve(self, maxit=10, fp=False, pn=default_fp_pn):
         # fp: bool, use fictious play or not
@@ -659,7 +659,7 @@ class MfgBase2D(object):
     def default_spi1_pn(n):
         return (n / (n + 2), 1 / (n + 2))
 
-    def solve_with_fp1(self, maxit=10, pn=default_spi1_pn):
+    def solve_with_spi1(self, maxit=10, pn=default_spi1_pn):
         self.diff_M = []
         # init
         QL1_bar = np.copy(self.QL1)
@@ -673,7 +673,7 @@ class MfgBase2D(object):
             # update U with M and Q_bar
             self.solve_HJB(self.M, Q_bar)
             # update Q with U and M
-            self.update_Q(self.U, self.M_bar)
+            self.update_Q(self.U, self.M)
             # update Q_bar
             p1, p2 = pn(i)
             QL1_bar = p1 * QL1_bar + p2 * self.QL1
@@ -692,9 +692,9 @@ class MfgBase2D(object):
         if n == 0:
             return (0, 1)
         else:
-            return (1 - 1 / (n+1), 1 / (n+1))
+            return (1 - 1 / (n + 1), 1 / (n + 1))
 
-    def solve_with_fp2(self, maxit=10, pn=default_spi2_pn):
+    def solve_with_spi2(self, maxit=10, pn=default_spi2_pn):
         self.diff_M = []
         # init
         QL1_hat = np.copy(self.QL1)
@@ -828,7 +828,7 @@ class MfgBase2D(object):
             ql2 = np.maximum((self.DL2 @ u) / self.F1_func(m), 0.0)
             qr2 = np.minimum((self.DR2 @ u) / self.F1_func(m), 0.0)
             q2 = ql1**2 + qr1**2 + ql2**2 + qr2**2
-            cost += np.sum(self.dt * self.dx**2 * m * 0.5 * self.F1_func(m) * q2)
+            cost += np.sum(self.dt * self.dx**2 * m * (0.5 * self.F1_func(m) * q2 + self.F2_func(m)))
         cost += np.sum(self.dx**2 * self.UT * self.M[-1, :])
         return cost
 
